<?php

namespace Nwb\Mail;

use InvalidArgumentException;

class Email
{
    const PRIORITY_HIGH = 'high';
    const PRIORITY_NORMAL = 'normal';
    const PRIORITY_LOW = 'low';

    private array $from;
    private array $replyTo;
    private array $to;
    private array $cc;
    private array $cci;
    private string $priority = 'normal';
    private string $subject;
    private string $body;
    private array $attachments = [];

    public function getFrom(): array
    {
        return $this->from;
    }

    public function getReplyTo(): array
    {
        return $this->replyTo;
    }

    public function getTo(): array
    {
        return $this->to;
    }

    public function getCc(): array
    {
        return $this->cc;
    }

    public function getCci(): array
    {
        return $this->cci;
    }

    public function getPriority(): string
    {
        return $this->priority;
    }

    public function getSubject(): string
    {
        return $this->subject;
    }

    public function getBody(): string
    {
        return $this->body;
    }

    public function getAttachments(): array
    {
        return $this->attachments;
    }

    public function setFrom(string | array $from): static
    {
        $this->from = $this->buildAddress($from);

        return $this;
    }

    public function setReplyTo(string | array $replyTo): static
    {
        $this->replyTo = $this->buildAddress($replyTo);

        return $this;
    }

    public function setTo(string ...$to): static
    {
        foreach ($to as $address) {
            $this->to[] = $this->buildAddress($address);
        }

        return $this;
    }

    public function setCc(string ...$cc): static
    {
        foreach ($cc as $address) {
            $this->cc[] = $this->buildAddress($address);
        }

        return $this;
    }

    public function setCci(string ...$cci): static
    {
        foreach ($cci as $address) {
            $this->cci[] = $this->buildAddress($address);
        }

        return $this;
    }

    public function setPriority(string $priority): static
    {
        if (!in_array($priority, [self::PRIORITY_HIGH, self::PRIORITY_NORMAL, self::PRIORITY_LOW])) {
            throw new InvalidArgumentException($priority . ' is not a valid priority');
        }

        $this->priority = $priority;

        return $this;
    }

    public function setSubject(string $subject): static
    {
        if (empty($subject)) {
            throw new InvalidArgumentException('subject is empty');
        }

        $this->subject = $subject;

        return $this;
    }

    public function setBody(string $body): static
    {
        $this->body = $body;

        return $this;
    }

    public function attachFromPath(string $path, string $attachmentName = null, string $cid = null): static
    {
        if (!is_file($path)) {
            throw new InvalidArgumentException('the file ' . $path . ' does not exist');
        }

        $base64 = shell_exec('base64 ' . $path);

        $filename = $attachmentName ?: basename($path);

        $this->attachments[] = [
            'filename' => $filename,
            'content' => $base64,
            'cid' => $cid
        ];

        return $this;
    }

    private function buildAddress(string | array $address): array
    {
        if (!is_array($address)) {
            $array = ['address' => $address, 'name' => null];
        } else {
            $array = $address;
        }

        $mail = isset($array['address']) ? $array['address'] : null;

        if (!filter_var($mail, FILTER_VALIDATE_EMAIL)) {
            throw new InvalidArgumentException($mail . ' is not a valid email address');
        }

        return $array;
    }
}
