<?php

namespace Nwb\Mail;

use Exception;

/**
 * Client pour l'API Mail sender NwB
 *
 * @link https://mail-sender.kub.nwb.fr/api-docs/#/default/post_send_mail_smtp
 */
class MailSender
{
    private string $host;
    private int $port;
    private string $user;
    private string $pass;
    private string $loginAPI;
    private string $passwordAPI;

    public function __construct(string $host, int $port, string $user, string $pass, string $loginAPI, string $passwordAPI)
    {
        // SMTP
        $this->host = $host;
        $this->port = $port;
        $this->user = $user;
        $this->pass = $pass;

        $this->loginAPI = $loginAPI;
        $this->passwordAPI = $passwordAPI;
    }

    public function send(Email $email)
    {
        $APIConf = [
            'Smtp' => [
                'host' => $this->host,
                'port' => $this->port,
                'user' => $this->user,
                'pass' => $this->pass
            ],
            'Mail' => [
                'subject' => $email->getSubject(),
                'html' => $email->getBody(),
                'from' => $email->getFrom(),
                'replyTo' => $email->getReplyTo(),
                'to' => $email->getTo(),
                'cc' => $email->getCc(),
                'cci' => $email->getCci(),
                'id' => '',
                'priority' => $email->getPriority(),
                'attachments' => $email->getAttachments()
            ],
            'Logins' => [
                'login' => $this->loginAPI,
                'password' => $this->passwordAPI
            ]
        ];

        // Attention la configuration doit impérativement être encodée en UTF 8 sans quoi le POST sera vide
        $APIConfJson = json_encode($APIConf);

        if ($APIConfJson === false) {
            throw new Exception(
                'Exception nwb\mail\send().json_encode() : Vérifier que tous les champs de la variable à encoder en JSON sont bien en UTF 8'
            );
        }

        $ch = curl_init('https://mail-sender.kub.nwb.fr/send-mail-smtp');
        $stderr = fopen('php://temp', 'wb');

        try {
            $opts = [
                CURLOPT_POST => true,
                CURLOPT_HTTPHEADER => ['Content-Type:application/json'],
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_CONNECTTIMEOUT => 30,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_MAXREDIRS => 2,
                CURLOPT_POSTFIELDS => $APIConfJson,
                CURLOPT_VERBOSE => true,
                CURLOPT_STDERR => $stderr,
                CURLOPT_SSL_VERIFYHOST => 0,
                CURLOPT_SSL_VERIFYPEER => 0
            ];

            curl_setopt_array($ch, $opts);

            $response = curl_exec($ch);
            $errno = curl_errno($ch);
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            rewind($stderr);
            $log = stream_get_contents($stderr);

            if (!$response) {
                throw new MailSenderAPIException(
                    self::debugMessage(['reason' => 'Pas de réponse du serveur', 'curlLog' => $log])
                );
            }

            if ($httpCode !== 200) {
                throw new MailSenderAPIException(
                    self::debugMessage(['reason' => 'Erreur HTTP', 'curlLog' => $log, 'responseBody' => $response])
                );
            }

            if ($errno) {
                throw new MailSenderAPIException(
                    self::debugMessage(['reason' => 'Erreur cURL', 'curlLog' => $log, 'curlErrno' => $errno])
                );
            }

            return json_decode($response, true);
        } finally {
            curl_close($ch);
            fclose($stderr);
        }
    }

    private static function debugMessage(array $params): string
    {
        $message = '<h1>Debug cURL</h1>';

        if (isset($params['reason'])) {
            $message .= "<p>Raison : {$params['reason']}</p>";
        }

        if (isset($params['curlErrno'])) {
            $message .= '<h2>Erreur cURL</h2>';
            $message .= "<p>#{$params['curlErrno']} : " . curl_strerror($params['curlErrno']) . '</p>';
        }
        if (isset($params['curlLog'])) {
            $message .= '<h2>Log cURL</h2>';
            $message .= "<pre>{$params['curlLog']}</pre>";
        }
        if (isset($params['responseBody'])) {
            $message .= '<h2>Réponse du serveur</h2>';
            $message .= "<p>{$params['responseBody']}</p>";
        }

        return $message;
    }
}
